function ZSpellButton_SetCount(self, count)
	self.count = count or self.count or 99
	count = self.count
	local name = self:GetName()
	local i = 1
	local frame = getglobal(name..i)
	while frame do
		if i < count then
			frame:SetAttribute("showstates", "1")
			if self.hiding == 0 or self.parent:GetAttribute("state") == "1" then
				frame:Show()
			end
		else
			frame:SetAttribute("showstates", ";")
			frame:Hide()
		end
		i = i + 1
		frame = getglobal(name..i)
	end
end

function ZSpellButton_SetTextures(self)
	local name = self:GetName()
	local i = 0
	local frame
	if self.skip then
		i = 1
		frame = getglobal(name..i)
		if not frame then return end
	else
		frame = self
	end
	local spell, texture
	while frame do
		spell = frame:GetAttribute("spell")
		texture = GetSpellTexture(spell)
		frame.icon:SetTexture(texture)
		i = i + 1
		frame = getglobal(name..i)
	end
end

function ZSpellButton_SetSpells(self, tbl)
	self.spells = tbl or self.spells or {}
	tbl = self.spells
	local name = self:GetName()
	local i = 0
	local frame
	if self.skip then
		i = 1
		frame = getglobal(name..i)
		if not frame then return end
	else
		frame = self
	end
	local spell, texture
	for index=1, table.getn(tbl) do
		spell = tbl[index]
		texture = GetSpellTexture(spell or "")
		if texture then
			frame:SetAttribute("spell", spell)
			frame.icon:SetTexture(texture)
			i = i + 1
			frame = getglobal(name..i)
			if not frame then break end
		end
	end
end

function ZSpellButton_SetLayout(self, direction)
	self.direction = direction or self.direction or "RIGHT"
	direction = self.direction
	local position = {["TOP"]="BOTTOM", ["BOTTOM"]="TOP", ["LEFT"]="RIGHT", ["RIGHT"]="LEFT"}
	if not position[direction] then
		direction = "RIGHT"
		position = "LEFT"
	else
		position = position[direction]
	end
	local name = self:GetName()
	local i = 1
	local cur = getglobal(name..i)
	local prev = self
	while cur do
		cur:ClearAllPoints()
		cur:SetPoint(position, prev, direction, 0, 0)
		i = i + 1
		prev = cur
		cur = getglobal(name..i)
	end
end

function ZSpellButton_SetSize(self, size)
	self.size = size or self.size or 36
	size = self.size
	local frame = self
	local name = self:GetName()
	local i = 0
	while frame do
		frame:SetHeight(size)
		frame:SetWidth(size)
		frame.cooldown:SetModelScale(0.69*size/36)
		i = i + 1
		frame = getglobal(name..i)
	end
end

function ZSpellButton_SetAnchorEvent(self, event)
	self.anchorevent = event or self.anchorevent or "OnRightClick"
	event = self.anchorevent
	local frame = self
	local name = self:GetName()
	local i = 0
	while frame do
		if event == "Always" then
			frame:SetAttribute("newstate1", ";")
		else
			frame:SetAttribute("newstate1", "0")
		end
		i = i + 1
		frame = getglobal(name..i)
	end
	self:SetAttribute("*childstate-OnEnter", ";")
	self:SetAttribute("*childstate-OnLeave", ";")
	self:SetAttribute("newstate2", ";")
	if event == "OnMouseOver" then
		self:SetAttribute("*childstate-OnEnter", "enter")
		self:SetAttribute("*childstate-OnLeave", "leave")
	elseif event == "OnRightClick" then
		self:SetAttribute("newstate2", "0:1;1:0")
	elseif event == "Always" then
		self.parent:SetAttribute("state", "1")
	end
end

function ZSpellButton_SetupButton(frame)
	frame.ismain = true
	frame.parent = CreateFrame("Frame", nil, UIParent, "SecureStateHeaderTemplate")
	frame:SetAttribute("anchorchild", frame.parent)
	frame.parent:ClearAllPoints()
	frame.parent:SetPoint("CENTER", frame, "CENTER", 0, 0)
	frame:SetAttribute("*childraise-OnEnter", true)
	frame.parent:SetAttribute("statemap-anchor-enter", "1")
	frame.parent:SetAttribute("statemap-anchor-leave", ";")
	frame.parent:SetAttribute("delaystatemap-anchor-leave", "1:0")
	frame.parent:SetAttribute("delaytimemap-anchor-leave", "1:0.4")
	frame.parent:SetAttribute("delayhovermap-anchor-leave", "1:true")
	local name = frame:GetName()
	local i = 0
	if frame.skip then
		frame:SetAttribute("newstate1", ";")
		frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		frame.parent:SetAttribute("AddChild", frame)
		i = 1
	end
	for index=1, table.getn(frame.spells) do
		local spell = frame.spells[index]
		local texture = GetSpellTexture(spell)
		if texture then
			local b
			if i == 0 then
				b = frame
			else
				b = CreateFrame("Button", name..i, nil, "ZSpellButtonTemplate")
			end
			b.icon:SetTexture(texture)
			b:SetAttribute("newstate1", ";")
			b:SetAttribute("type1", "spell")
			b:SetAttribute("spell", spell)
			if i ~= 0 then
				b:SetAttribute("showstates", "1")
			end
			b:RegisterForClicks("LeftButtonUp", "RightButtonUp")
			frame.parent:SetAttribute("AddChild", b)
			i = i + 1
		end
	end
	if i == 0 then
		frame:UnregisterAllEvents()
		frame:Hide()
		return
	end
	hooksecurefunc(getmetatable(frame).__index, "SetAttribute", ZSpellButton_OnEnter)
	frame:ZSetLayout()
	frame:ZSetSize()
	frame:ZSetAnchorEvent()
	frame:ZSetCount()
end

function ZSpellButton_CreateObjects(self)
	self.cooldown = CreateFrame("Model", nil, self, "CooldownFrameTemplate")
	self.cooldown:SetAllPoints(self)
	self.cooldown:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	self.cooldown:SetScript("OnEvent", ZSpellButton_Cooldown)
	self.cooldown:SetModelScale(0.69)
	self.icon = self:CreateTexture(nil, "LOW")
	self.icon:SetAllPoints(self)
end

function ZSpellButton_OnLoad(self)
	ZSpellButton_AddFunctions(self)
	self:RegisterForDrag("LeftButton")
	self:RegisterEvent("PLAYER_LOGIN")
	self:RegisterEvent("VARIABLES_LOADED")
end

function ZSpellButton_OnEvent(self)
	if event == "PLAYER_LOGIN" then
		if type(self.spells) == "table" then
			self:UnregisterEvent("PLAYER_LOGIN")
			ZHunterSetupAnchor(self)
		end
	end
end

function ZSpellButton_OnEnter(self, att, val)
	if att == "_entered" then
		if not val then
			GameTooltip:Hide()
			return
		end
	elseif att then
		return
	end
	local spell = self:GetAttribute("spell") or self.ztip
	if spell then
		local anchor = "ANCHOR_TOPLEFT"
		local temp = self.direction or self:GetPoint(1)
		if temp == "BOTTOM" or temp == "TOP" then
			anchor = "ANCHOR_RIGHT"
		end
		GameTooltip:SetOwner(self, anchor)
		GameTooltip:SetText(spell)
	end
end

function ZSpellButton_OnLeave()
	GameTooltip:Hide()
end

function ZSpellButton_Cooldown(self)
	local spell = self:GetParent():GetAttribute("spell")
	if not spell then return end
	local start, duration, enable = GetSpellCooldown(spell)
	CooldownFrame_SetTimer(self, start, duration, enable)
end

function ZSpellButton_AddFunctions(self)
	self.ZSetTextures = ZSpellButton_SetTextures
	self.ZSetLayout = ZSpellButton_SetLayout
	self.ZSetSize = ZSpellButton_SetSize
	self.ZSetAnchorEvent = ZSpellButton_SetAnchorEvent
	self.ZSetCount = ZSpellButton_SetCount
	self.ZSetSpells = ZSpellButton_SetSpells
end